/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.generator.idl.lib;

/** To access all the AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access all the AST Scope. */
import org.objectweb.corba.ast.api.Scope;

/** To access Generation Exception. */
import org.objectweb.corba.generator.common.lib.GenerationException;

/** To use ArrayList. */
import java.util.ArrayList;

/** To use Vector. */
import java.util.Vector;


public class IDL3Generator
     extends org.objectweb.corba.generator.common.lib.GeneratorBase
  implements org.objectweb.corba.generator.idl.api.IDL3Generator
{

    // ===========================================================
    //
    // Internal State.
    //
    // ===========================================================

    /**
     ** Utility class to convert types in IDL
     **/
    public org.objectweb.corba.generator.idl.lib.IDLTranslator translator_;

    /**
     ** The last prefix defined in the file
     **/
    protected java.lang.String last_prefix_;

    /**
     ** Print the macro directive ?
     **/
    protected boolean print_macro_;


    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */

    public IDL3Generator(org.objectweb.corba.ast.api.AST ast)
    {
        // Call the GeneratorBase constructor
        super(ast);

        // Init internal states
        translator_ = new org.objectweb.corba.generator.idl.lib.IDLTranslator();
        last_prefix_ = "";
        print_macro_ = true;
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Get the Macro Name from a declaration.
     *
     * @param target_decl - The scope to visit.
     *
     * @return The macro name.
     */
    protected String
    get_macro(Scope target_decl)
    {
        int idx, idx2;
        String abs_name = null,
               res = "";
        try{
            abs_name = target_decl.getAbsoluteName();

            idx = abs_name.indexOf(':', 2);
            idx2 = 2;
            while (idx!=-1)
            {
                res = res + abs_name.substring(idx2, idx) + '_';
                idx2 = idx+2;
                idx = abs_name.indexOf(':', idx2);
            }
            res = res + abs_name.substring(idx2);
            res = "__" + res.toUpperCase() + "__";
        }catch(Exception e){
            res = "__NO_NAME__";
            print_macro_ = false;
        }
        return res;
    }

    /**
     * Method called after generation.
     * Generally used to close files.
     **/
    protected void
    after_generation()
    {
        close("out");
    }


    // ===========================================================
    //
    // Public methods.
    //
    // ===========================================================

    /**
     * Initialize the generator.
     *
     * @param outputfile - The name where declarations will be generated.
     * @param imports - IR declarations to import.
     * @param app_name - The application's name.
     **/
    public void
    initialize( String outputfile,
                ArrayList imports,
                String app_name )
    {
        Vector vect = new Vector();

        /** Initialize environment **/

        // The ressource path
        vect.add(System.getProperty("ccm.templates"));
        setRessourcePath(vect);

        // Templates to use.
        vect.clear();
        vect.add("common.vm");
        vect.add("idl3.vm");
        setLibrary(vect);

        // The output file
        open(outputfile, "out");

        // call the common generator init method
        super.init();

        /** Initialize velocity context **/
        put("which", app_name);
        put("gen", this);
        put("declarator", "");
        put("limited_types" , new Long(org.objectweb.corba.ast.api.DeclarationKind.dk_all));
        put("imports", imports);
    }

    /**
     * Generates contents of a scope.
     * name must be a valid declaration name.
     *
     * @param name - The name of the declaration to visit.
     *
     * @throws GenerationException
     **/
    public void
    generate(String name)
    throws GenerationException
    {
        Scope target_decl = null;

        try{
            target_decl = (Scope)getDeclaration(name);
        }catch(ClassCastException ex){
            String msg = name + "is not a scope!";
            throw new GenerationException(msg);
        }
        generate(target_decl);
    }

    /**
     * Generates contents of a scope.
     *
     * @param target_decl - The scope to visit.
     *
     * @throws GenerationException
     **/
    public void
    generate(Scope target_decl)
    throws GenerationException
    {
        target_decl_ = target_decl;

        // System.out.println("### Visiting scope (generate) :  " + target_decl.getAbsoluteName());
        put("obj", target_decl);
        put("FILE_MACRO", get_macro(target_decl));
        map("FILE_HEADER");
        visit(target_decl);
        after_generation();
    }

    /**
     * Generates contents of the root scope.
     * Declarations types must match to limited types.
     *
     * @param target_decl - The scope to visit.
     * @param limited_types - A logical combination of DeclarationKind.
     *
     * @throws GenerationException
     **/
    public void
    generate(Scope target_decl, long limited_types)
    throws GenerationException
    {
        Vector vect = null;

        vect = getDeclarations(target_decl, limited_types);
        put("limited_types" , new Long(limited_types));
        put("FILE_MACRO", get_macro(target_decl));
        map("FILE_HEADER");
        forward(vect);
        map("END_LINE");
        visit(vect);
        after_generation();
    }

    /**
     * Close an idl stream.
     *
     * @param id - The identifiant of the file to close.
     */
    public void
    close(String id)
    {
        if ( print_macro_ )
            print("\n#endif", id);
        super.close(id);
    }

    // ===========================================================
    //
    // Useful methods for IDL3 Mapping.
    //
    // ===========================================================

    /**
     * Utility method to check forward declarations.
     *
     * @return The associated search mask.
     **/
    public long
    forward_check()
    {
        return (DeclarationKind.dk_interface +
                DeclarationKind.dk_local_interface +
                DeclarationKind.dk_abstract_interface +
                DeclarationKind.dk_component +
                DeclarationKind.dk_value +
                DeclarationKind.dk_struct +
                DeclarationKind.dk_union +
                DeclarationKind.dk_event);
    }

    /**
     * Utility method to check interface declarations.
     *
     * @return The associated search mask.
     **/
    public long
    interface_check()
    {
        return (DeclarationKind.dk_attribute +
                DeclarationKind.dk_constant +
                DeclarationKind.dk_exception +
                DeclarationKind.dk_operation +
                DeclarationKind.dk_alias +
                DeclarationKind.dk_struct +
                DeclarationKind.dk_union +
                DeclarationKind.dk_enum +
                DeclarationKind.dk_native);
    }

    /**
     * Utility method to check component declarations.
     *
     * @return The associated search mask.
     **/
    public long
    component_check()
    {
        return (DeclarationKind.dk_provides +
                DeclarationKind.dk_uses +
                DeclarationKind.dk_emits +
                DeclarationKind.dk_publishes +
                DeclarationKind.dk_consumes +
                DeclarationKind.dk_attribute);
    }

    /**
     * Utility method to check home declarations.
     *
     * @return The associated search mask.
     **/
    public long
    home_check()
    {
        return (DeclarationKind.dk_factory +
                DeclarationKind.dk_finder +
                interface_check());
    }

    /**
     * Utility method to check value declarations.
     *
     * @return The associated search mask.
     **/
    public long
    value_check()
    {
        return (DeclarationKind.dk_value_member +
                interface_check());
    }

    /**
     * Get an attribute of the class DeclarationKind.
     *
     * @param att - The attribute to get.
     *
     * @return The double value of the attribute.
     **/
    public long
    getDeclarationKind(String att)
    {
        java.lang.reflect.Field field = null;
        long res = 0;

        try{
            field = Class.forName("org.objectweb.corba.ast.api.DeclarationKind").getField(att);
            res = field.getLong(field);
        }catch(Exception e){
            e.printStackTrace();
        }

        return res;
    }

    // ===========================================================
    //
    // Methods for IDL3 Mapping.
    //
    // ===========================================================

    /**
     ** Generate a macro if necessary
     **/
    public String
    macro(Declaration obj)
    {
        String res = "";
        String version = obj.getVersion();
        String id = obj.getId();
        String prefix = obj.getPrefix();

        // System.err.println("Macro for " + obj.getAbsoluteName() + " : " + prefix);
        if (!prefix.equals(last_prefix_))
        {
            res = res + "#pragma prefix \""+prefix+"\"\n";
            last_prefix_ = prefix;
        }
        else
        {
            String p_id = obj.getParent().getId();
            int idx1 = p_id.indexOf(':');
            int idx2 = p_id.lastIndexOf(':');
            if (id.indexOf(p_id.substring(idx1, idx2))==-1)
                res = res + "#pragma id "+obj.getName()+" \""+id+"\"\n";
        }
        if (!version.equals("1.0"))
            res = res + "#pragma version "+obj.getName()+" "+version+"\n";

        return res;
   }

    /**
     * Generate structured members contained in a declaration
     * (i.e. a struct, an exception or a declaration).
     *
     * @param obj - The declaration to visit
     **/
    public void
    struct_members(Declaration obj, String obj_declarator)
    {
        boolean is_inner = false;
        int multiple_idx = -1;
        org.objectweb.corba.ast.api.WithNameTypeRef[] members = new org.objectweb.corba.ast.api.WithNameTypeRef[0];

        // Retrieve members of the declaration
        try
        {
            members = ((org.objectweb.corba.ast.api.ExceptionDecl)obj).getMemberList().getStructMembers();
        }catch(ClassCastException e){
            try{
                members = ((org.objectweb.corba.ast.api.StructDecl)obj).getMemberList().getStructMembers();
            }catch(ClassCastException ex){
                members = ((org.objectweb.corba.ast.api.UnionDecl)obj).getMemberList().getUnionMembers();
            }
        }
//        System.err.println("members de struct_members : " + members);

        // Compute each member
        for (int i=0; i<members.length; i++)
        {
            String name = members[i].getName();
            org.objectweb.corba.ast.api.TypeRef type = members[i].getType();

            // check for inner struct, union or enum members
            if ((type.getTypeKind() == org.objectweb.corba.ast.api.TypeKind.tk_struct) ||
                (type.getTypeKind() == org.objectweb.corba.ast.api.TypeKind.tk_union) ||
                (type.getTypeKind() == org.objectweb.corba.ast.api.TypeKind.tk_enum))
            {
                Declaration decl = null;
                decl = (Declaration)type;
                if (decl.getParent().getId().equals(obj.getId()))
                    is_inner = true;
                else
                    is_inner = false;
            }
            else
                is_inner = false;


            // compute the declarator
            String declarator = "";
            // check for multiple declarators
            //  e.g.  long l1, l2, l3;
            if ( (i<members.length-1) && (type==members[i+1].getType()) )
                multiple_idx = i;
            while ( (i<members.length-1) && (type==members[i+1].getType()) )
                i++;

            if (multiple_idx==-1)
                declarator = name + translator_.toIDLPostfix(type);
            else
            {
                for (int j=multiple_idx; j<i+1; j++)
                    declarator = declarator + members[j].getName() + translator_.toIDLPostfix(members[j].getType()) + ", ";
                declarator = declarator.substring(0, declarator.length()-2);
                multiple_idx = -1;
            }

            if (is_inner)
            {
                // inline struct, union or enum
                Declaration decl = null;
                String id = "";

                decl = (Declaration)type;
                id = org.objectweb.corba.ast.lib.DeclarationKindImpl.toString(decl.getDeclKind()).toUpperCase()+ "_INLINED";

                put("obj", decl);
                put("declarator", declarator);
                map(id);
            }
            else
            {
                put("type_spec", translator_.toIDL(type) );
                put("name", declarator);
                map("STRUCT_MEMBER");
            }
        }//end for

        put("declarator",obj_declarator);
        put("obj", obj);
    }


    /**
     * Generate union members for an union.
     *
     * @param union - The union to visit
     **/
    public void
    union_members(org.objectweb.corba.ast.api.UnionDecl union, String obj_declarator)
    {
        org.objectweb.corba.ast.api.UnionMember[] members = null;
        boolean is_default = false;
        boolean print = true;
        boolean is_inner = false;
        int di = union.getDefaultIndex();

        members = union.getMemberList().getUnionMembers();

//union.getContents(true, DeclarationKind.dk_all);
//System.err.println("default index:"+di);
        for (int i=0; i<members.length; i++)
        {
            String name = members[i].getName();
            org.objectweb.corba.ast.api.TypeRef type = members[i].getType();
            org.objectweb.corba.ast.api.AnyValue val = members[i].getAnyValue();

            // check that the i and i+1 element spec are different.
            // else just print the label
            if ( ( i<members.length-1 ) &&
                 ( name.equals(members[i+1].getName()) )
                )
                print = false;
            else
                print = true;

// TODO : Correct the bug with default index (always -1)
//            if (di == i)
            if (translator_.toString(val, union.getDiscriminator()).equals(""))
                is_default = true;
            else
                is_default = false;
//System.err.println("Default="+is_default+", i:"+i);

            // check for inner struct, union or enum members
            if ( (type.getTypeKind() == org.objectweb.corba.ast.api.TypeKind.tk_struct) ||
                 (type.getTypeKind() == org.objectweb.corba.ast.api.TypeKind.tk_union) ||
                 (type.getTypeKind() == org.objectweb.corba.ast.api.TypeKind.tk_enum))
            {
                Declaration decl = null;
                decl = (Declaration)type;
                if (decl.getParent().getId().equals(union.getId()))
                    is_inner = true;
                else
                    is_inner = false;
            }
            else
                is_inner = false;

            if (!print)
            {
                // print the label
                if (is_default)
                    print("default :\n");
                else
                {
                    String value = translator_.toString(val, union.getDiscriminator());
                    print("case " + value + " :\n");
                }
            }
            else if (is_inner)
            {
                // print the label first
                if (is_default)
                    print("default :\n");
                else
                {
                    String value = translator_.toString(val, union.getDiscriminator());
                    print("case " + value + " :\n");
                }

                // inline struct, union or enum
                Declaration decl = null;
                String id = "";

                decl = (Declaration)type;
                id = org.objectweb.corba.ast.lib.DeclarationKindImpl.toString(decl.getDeclKind()).toUpperCase()+
                    "_INLINED";

                put("obj", decl);
                put("declarator", name+translator_.toIDLPostfix(type));
                // reajust the tabulation
                //print("/inc");
                map(id);
                //print("/dec");
            }
            else
            {
                put("value", translator_.toString(val, union.getDiscriminator()));
                put("type_spec",translator_.toIDL(type));
                put("name", name+translator_.toIDLPostfix(type));
                if (is_default)
                    map("UNION_DEFAULT_MEMBER");
                else
                    map("UNION_MEMBER");
            }
        }//end for
        put("declarator",obj_declarator);
        put("obj", union);
   }


    /**
     * Generate members of an enumeration
     *
     * @param enum - The enumeration.
     **/
    public void
    enum_members(org.objectweb.corba.ast.api.EnumDecl enum)
    {
        String[] members = enum.getMemberList().getStrings();
        String res = "";
        int i;

        for (i=0; i<members.length-1; i++)
            print( members[i] + ",\n");

        if (i<members.length)
            print( members[i] );
    }


    /**
     * Generate the modifier of a ValueDecl
     *
     * @param value - The ValueDecl concerned.
     **/
    public void
    modifier(org.objectweb.corba.ast.api.ValueDecl value)
    {
        if (value.isCustom())
            print("custom ");
        else if (value.isAbstract())
            print("abstract ");
    }


    /**
     * Generate inherited component for a component.
     *
     * @param comp - The component concerned.
     *
     * @return The resulting string.
     **/
    public String
    base(org.objectweb.corba.ast.api.ComponentDecl comp)
    {
        org.objectweb.corba.ast.api.ComponentDecl inh = comp.getBaseComponent();
        if (inh != null)
            return (" : " + inh.getAbsoluteName());
        return "";
   }

    /**
     * Generate inherited home.
     *
     * @param home - The home concerned.
     *
     * @return The resulting string.
     **/
    public String
    base(org.objectweb.corba.ast.api.HomeDecl home)
    {
        org.objectweb.corba.ast.api.HomeDecl inh = home.getBaseHome();
        if (inh != null)
            return (" : "+ inh.getAbsoluteName());
        return "";
   }


    /**
     * Generate supported interfaces.
     *
     * @param comp - The list of interaces concerned.
     *
     * @return The resulting string.
     **/
    public String
    supports(org.objectweb.corba.ast.api.InterfaceDecl[] itfs)
    {
        String res = "";

        if ( (itfs!=null) && (itfs.length>0) )
        {
            res += "\n          supports ";
            int i;
            for (i=0; i<itfs.length-1; i++)
                res += itfs[i].getAbsoluteName() + ", ";
            res += itfs[i].getAbsoluteName();
        }
        return res;
  }

    /**
     * Generate supported interfaces for a component base.
     *
     * @param comp - The component base concerned.
     *
     * @return The resulting string.
     **/
    public String
    supported(org.objectweb.corba.ast.api.ComponentBase comp)
    {
        org.objectweb.corba.ast.api.InterfaceList list = null;

        list = comp.getSupportedInterfaceList();
        if (list != null)
            return supports( list.getInterfaces() );
        return "";
    }

    /**
     * Generate supported interfaces for a value declaration.
     *
     * @param comp - The value declaration concerned.
     *
     * @return The resulting string.
     **/
    public String
    supported(org.objectweb.corba.ast.api.ValueDecl value)
    {
        org.objectweb.corba.ast.api.InterfaceList list = null;

        list = value.getSupportedInterfaceList();
        if (list != null)
            return supports( list.getInterfaces() );
        return "";
    }

    /**
     * Generate inherited interfaces for an interface.
     *
     * @param itf - The interface concerned.
     *
     * @return The string format.
     **/
    public String
    bases(org.objectweb.corba.ast.api.InterfaceDecl itf)
    {
        String res = "";
        Declaration[] itfs = itf.getInheritedInterfaceList().getInterfaces();

        if (itfs.length>0)
            res += "\n              : ";

        for (int i=0; i<itfs.length; i++)
        {
            res += itfs[i].getAbsoluteName();
            if (i<itfs.length-1)
                res += ",\n                " ;
        }
        return res;
    }

    /**
     * Generate mode for an operation.
     *
     * @param op - The operation concerned.
     *
     * @return The string format.
     **/
    public String
    mode(org.objectweb.corba.ast.api.OperationDecl op)
    {
        if (op.isOneway())
            return("oneway ");
        else
            return "";
    }

    /**
     * Convert a parameter mode in String format
     *
     * @param param - The parameter concerned.
     *
     * @return The string format.
     **/
    public String
    toStringMode(org.objectweb.corba.ast.api.Parameter param)
    {
        if ( param.isIn() )
            return "in ";
        else if ( param.isOut() )
            return "out ";
        else
            return "inout ";
    }

    /**
     * Generate type for a type reference.
     *
     * @param obj - The type reference concerned.
     *
     * @return The string format.
     **/
    public String
    type(org.objectweb.corba.ast.api.TypeRef obj)
    {
        return translator_.toIDL(obj);
    }

    /**
     * Generate parameters for an operation.
     *
     * @param op - The operation concerned.
     *
     * @return The string format.
     **/
    public String
    params(org.objectweb.corba.ast.api.OperationDecl op)
    {
        String res = "";
        org.objectweb.corba.ast.api.Parameter[] par = op.getParameterList().getParameters();

        for (int i=0; i<par.length; i++)
            res += toStringMode(par[i]) + translator_.toIDL(par[i].getType()) + " " + par[i].getName() + ", ";

        if (!res.equals(""))
            res = res.substring(0, res.length()-2);

        return res;
    }

    /**
     * Generate exceptions raises.
     *
     * @param exc - The list of exceptions concerned.
     *
     * @return The string format.
     **/
    public String
    raise(String raises_format, org.objectweb.corba.ast.api.ExceptionList list)
    {
        org.objectweb.corba.ast.api.ExceptionDecl[] excs = list.getExceptions();
        String res = "";

        if ( (excs != null) && (excs.length>0))
        {
            int i;

            res += ("\n   " + raises_format + "(");
            for (i=0; i<excs.length-1; i++)
                res += excs[i].getAbsoluteName() + ", ";
            res += excs[i].getAbsoluteName() + ")";
        }
        return res;
    }

    /**
     * Generate exceptions raises for an attribute.
     *
     * @param attr - The attribute concerned.
     *
     * @return the string format.
     **/
    public String
    raises(org.objectweb.corba.ast.api.AttributeDecl attr)
    {
        return raise("raises", attr.getExceptionList());
    }

    /**
     * Generate getexceptions raises for an attribute.
     *
     * @param attr - The attribute concerned.
     *
     * @return The string format.
     **/
    public String
    getraises(org.objectweb.corba.ast.api.AttributeDecl attr)
    {
        return raise("getraises", attr.getGetExceptionList());
    }

    /**
     * Generate getexceptions raises for an attribute.
     *
     * @param attr - The attribute concerned.
     *
     * @return the string format.
     **/
    public String
    setraises(org.objectweb.corba.ast.api.AttributeDecl attr)
    {
        return raise("setraises", attr.getSetExceptionList());
    }

    /**
     * Generate access mode for an attribute.
     *
     * @param attr - The attribute concerned.
     *
     * @return the string format.
     **/
    public String
    access(org.objectweb.corba.ast.api.AttributeDecl attr)
    {
        if (attr.isReadonly())
            return "readonly ";
        return "";
    }

    /**
     * Generate multiple use declaration.
     *
     * @param uses - The use declaration concerned.
     *
     * @return The string format.
     **/
    public String
    multiple(org.objectweb.corba.ast.api.UsesDecl uses)
    {
        if (uses.isMultiple())
            return "multiple ";
        else
            return "";
    }

    /**
     * Generate primary key for a home.
     *
     * @param home - The home concerned.
     *
     * @return The string format.
     **/
    public String
    primary_key(org.objectweb.corba.ast.api.HomeDecl home)
    {
        Declaration pk = home.getPrimaryKey();

        if (pk == null)
            return "";
        else
            return ("\n     primary key " + pk.getAbsoluteName());
   }

    /**
     * Generate base value for a value declaration.
     *
     * @param value - The value declaration concerned.
     *
     * @return the string format.
     **/
    public String
    base_value(org.objectweb.corba.ast.api.ValueDecl value)
    {
        org.objectweb.corba.ast.api.ValueDecl base = value.getBaseValue();
        if (base!=null)
        {
            if (value.isTruncatable())
                return " : truncatable " + base.getAbsoluteName();
            else
                return " : " + base.getAbsoluteName();
        }
        return "";
   }


    /**
     * Generate abstract values for a value declaration.
     *
     * @param value - The value declaration concerned.
     *
     * @return The resulting string.
     **/
    public String
    abstract_values(org.objectweb.corba.ast.api.ValueDecl value)
    {
        org.objectweb.corba.ast.api.ValueDecl[] bases = value.getAbstractValueList().getValues();
        String res = "";

        if (bases.length>0)
        {
            if (value.getBaseValue() == null)
                res += " : ";
            else
                res += ", ";
            int i;
            for (i=0; i<bases.length-1; i++)
                res += bases[i].getAbsoluteName() + ", ";
            res += bases[i].getAbsoluteName();
        }
        return res;
    }

    /**
     * Generate visibility for a value member.
     *
     * @param vm - The value member concerned.
     *
     * @return The string format.
     **/
    public String
    visibility(org.objectweb.corba.ast.api.ValueMemberDecl vm)
    {
        if (vm.isPrivate())
            return "private";
        else
            return "public";
    }

    /**
     * Translate an array into its idl format.
     *
     * @param obj - The array to translate.
     *
     * @return The resulting string.
     **/
    public String
    array(org.objectweb.corba.ast.api.TypeRef obj)
    {
        return translator_.toIDLPostfix(obj);
    }

    /**
     * Generate contexts for an operation.
     *
     * @param op - The operation concerned.
     *
     * @return The string format.
     **/
    public String
    contexts(org.objectweb.corba.ast.api.OperationDecl op)
    {
        String[] ctxs = op.getContextList().getStrings();
        String res = "";

        if (ctxs.length > 0)
        {
            res += " context(";
            for (int i=0; i<ctxs.length; i++)
                res += "\"" + ctxs[i] + "\", ";
            res += ")";
        }

        return res;
    }


    /**
     * Generate value for a constant.
     *
     * @param constant - The constant declaration concerned.
     *
     * @return The string format.
     **/
    public String
    value(org.objectweb.corba.ast.api.ConstantDecl constant)
    {
        return translator_.toString(constant.getAnyValue(), constant.getType());
    }


    /**
     * Generate initializers parameters for an initializer.
     *
     * @param obj - The initializer concerned.
     *
     * @return The resulting string.
     **/
    public String
    initializer_params(org.objectweb.corba.ast.api.Initializer obj)
    {
        org.objectweb.corba.ast.api.Parameter[] params = null;
        params = obj.getParameterList().getParameters();
        String res = "";

        for (int i=0; i<params.length; i++)
        {
            res += toStringMode(params[i]);
            res += translator_.toIDL(params[i].getType());
            res += " " + params[i].getName() + ", ";
        }

        if (!res.equals(""))
            res = res.substring(0, res.length()-2);

        return res;
    }


    /**
     * Generate initializers (i.e. factory) for a value Declaration.
     *
     * @param value - The value declaration concerned.
     **/
    public void
    initializers(org.objectweb.corba.ast.api.ValueDecl value)
    {
        org.objectweb.corba.ast.api.Initializer[] ops = value.getInitializerList().getInitializers();

        for (int i=0;i<ops.length;i++)
        {
            put("init", ops[i]);
            map("INITIALIZER");
        }
    }


}
