/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.api;

/**
 * Naming Context Abstraction.
 *
 * API not stable, should be strongly discuted.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 *
 * Note: This is inspired from the Jonathan naming context framework
 * initially developped by Fabien Delpiano and Bruno Dumant.
 */

public interface Context {
	
    /**
     * The standard name of this interface to use
     * in the context of the Fractal framework.
     */
    static final String CONTEXT = Context.class.getName();

    /**
     * Gets the scope of the target context.
     *
     * This method does not update the scope of the target context.
     *
     * @return The scope of the target context, or null if the
     *         target context has no scope.
     *
     * @postcondition return == getOWScope();
     */
    Context getOWScope();

    /**
     * Gets a local entry in the target context identified by <code>name</code>,
     * or null if no entry exists under name <code>name</code>.
     *
     * Here is a local resolution in the target context.
     *
     * This method does not modify the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *      result.getOWName().getOWId().equals(name)
     *  and result.getOWContext() == this
     *  and this.isNotModified();
     */
    Entry getOWLocalEntry(String name);

    /**
     * Gets the value of a local entry in the target context identified by
     * <code>name</code>, or null if no entry exists under name
     * <code>name</code>.
     *
     * This method is equivalent to
     * {@link #getOWLocalEntry(String) <code>getOWLocalEntry(name)</code>}
     * {@link Entry#getOWValue() <code>.getOWValue()</code>}.
     *
     * This method does not modify the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry value, if any; or null otherwise.
     *
     * @postcondition
     *     ( getOWLocalEntry(name) == null && result == null )
     *  or result == getOWLocalEntry(name).getOWValue());
     */
    Object getOWLocalValue(String name);

    /**
     * Gets the entry in the target context identified by <code>name</code>,
     * or null if no entry exists under name <code>name</code>.
     *
     * This method does not modify the given name and the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *      result.getOWName().equals(name)
     *  and name.isNotModified()
     *  and this.isNotModified();
     */
    Entry getOWEntry(Name name);

    /**
     * Gets the value of an entry in the target context identified by
     * <code>name</code>, or null if no entry exists under name
     * <code>name</code>.
     *
     * This method is equivalent to
     * {@link #getOWEntry(Name) <code>getOWEntry(name)</code>}
     * {@link Entry#getOWValue() <code>.getOWValue()</code>}.
     *
     * This method does not modify the given name and the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *     ( getOWEntry(name) == null && result == null )
     *  or result == getOWEntry(name).getOWValue());
     */
    Object getOWValue(Name name);

    /**
     * Returns a new {@link EntryIterator EntryIterator} to
     * iterate on the entries contained by the target context.
     *
     * @return A new EntryIterator.
     */
    EntryIterator getOWEntries();

}
