/*===========================================================================

ObjectWeb Util API
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.wrappers.common;

/** To use the ExceptionHolder interface */
import org.objectweb.util.browser.naming.api.ExceptionHolder;

/**
 * RuntimeException subclass to wrap a Java exception.
 *
 * As ExceptionWrapper is a subclass of RuntimeException,
 * a method is not required to declare it in its throws clause
 * that might be thrown during the execution of the method but not caught.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */

public class ExceptionWrapper
    extends java.lang.RuntimeException
    implements ExceptionHolder {
    	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The wrapped Java exception. */
    private Exception exception_ = null;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Constructs an ExceptionWrapper with no detail message and
     * no wrapped Java exception.
     *
     * This constructor sets the detail message and
     * the wrapped Java exception to the null value.
     *
     * @postcondition
     *          getMessage() == null
     *  and getOWException() == null;
     */
    public ExceptionWrapper() {
        this(null, null);
    }

    /**
     * Constructs an ExceptionWrapper with a detail message and
     * no wrapped Java exception.
     *
     * This constructor sets the detail message to the given message
     * and the wrapped Java exception to the null value.
     *
     * @param message The detail message.
     *
     * @postcondition
     *          getMessage() == message
     *  and getOWException() == null;
     */
    public ExceptionWrapper(String message) {
        this(message, null);
    }

    /**
     * Constructs an ExceptionWrapper with no detail message and
     * a wrapped Java exception.
     *
     * This constructor sets the detail message to the null value
     * and the wrapped Java exception to the given exception.
     *
     * @param exception The Java exception to wrap.
     *
     * @postcondition
     *          getMessage() == null
     *  and getOWException() == exception;
     */
    public ExceptionWrapper(Exception exception) {
        this(null, exception);
    }

    /**
     * Constructs an ExceptionWrapper with a detail message and
     * a wrapped Java exception.
     *
     * This constructor sets the detail message to the given message
     * and the wrapped Java exception to the given exception.
     *
     * @param message The detail message.
     * @param exception The Java exception to wrap.
     *
     * @postcondition
     *          getMessage() == message
     *  and getOWException() == exception;
     */
    public ExceptionWrapper(String message, Exception exception) {
        // Calls the RuntimeException constructor.
        super(message);

        // Inits the internal state.
        setOWException(exception);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface ExceptionHolder.
    //
    // ==================================================================

    /**
     * Returns the wrapped Java exception.
     *
     * This method does not update the wrapped Java exception.
     *
     * @return The wrapped Java exception.
     *
     * @postcondition return == getOWException();
     */
    public java.lang.Exception getOWException() {
        return exception_;
    }

    /**
     * Sets the wrapped Java exception.
     *
     * This method sets the wrapped Java exception with the given exception.
     *
     * @param exception The Java exception to wrap.
     *
     * @postcondition getOWException() == exception;
     */
    public void setOWException(java.lang.Exception exception) {
        exception_ = exception;
    }
}
